// Library:       WebSphere MQ
// Component:     MQ.NET Sample Programs
// Part:          MMQWRLD.CPP
//
// Description:   Sample Managed C++ program that puts and gets a message
//                to a queue.
//
//                MMQWRLD has 3 parameters:
//                - the name of a queue (optional)
//                  e.g. SYSTEM.DEFAULT.LOCAL.QUEUE or
//                       SYSTEM.DEFAULT.MODEL.QUEUE
//                - the name of a queue manager (optional)
//                - a channel definition (optional)
//                  e.g. SYSTEM.DEF.SVRCONN/TCP/hostname(1414)
//
//                If no queue name is given, the name defaults to
//                SYSTEM.DEFAULT.LOCAL.QUEUE.
//
//                If no queue manager name is given, the queue manager
//                defaults to the default local queue manager.
//  <START_COPYRIGHT> 
//  Licensed Materials - Property of IBM 
//                   
//  5724-B41         
//  (C) Copyright IBM Corp. 1994, 2003 All Rights Reserved. 
//                   
//  US Government Users Restricted Rights - Use, duplication or 
//  disclosure restricted by GSA ADP Schedule Contract with 
//  IBM Corp.        
//  <END_COPYRIGHT> 

#using <mscorlib.dll>
#using <amqmdnet.dll>

#include <tchar.h>

using namespace System;
using namespace IBM::WMQ;

// This is the entry point for this application
int _tmain(int argc, char * * argv )
{
	String* EXISTING_QUEUE = "SYSTEM.DEFAULT.LOCAL.QUEUE";
	String* MESSAGE = "Hello world";

	MQQueueManager*      mqQMgr;          // MQQueueManager instance
	MQQueue*             mqQueue;         // MQQueue instance
	String*              queueName;       // Name of queue to use
	MQMessage*           mqMsg;           // MQMessage instance
	MQPutMessageOptions* mqPutMsgOpts;    // MQPutMessageOptions instance
	MQGetMessageOptions* mqGetMsgOpts;    // MQGetMessageOptions instance

	System::Console::WriteLine( S"Sample MMQWRLD start" );

	///
	/// Try to create an MQQueueManager instance 
	/// 
	try 
	{
		if (argc > 3) 
		{
			// queue name, queue manager name, channel definition all provided

			// Break down the channel definition,
			// which is of the form "channel-name/transport-type/connection-name".
			String* channelDefinition = argv[3];
		    String* channelName = "";
			String* transportType = "";
			String* connectionName = "";
			Char separator[] = {'/'};
			String* parts[];
			parts = channelDefinition->Split( separator );
			if (parts.Length > 0) channelName = parts[0];
			if (parts.Length > 1) transportType = parts[1];
			if (parts.Length > 2) connectionName = parts[2];
			mqQMgr = new MQQueueManager( argv[2], channelName, connectionName );
		}
		else
			if (argc == 3) 
		{
			// queue name, queue manager name provided
			mqQMgr = new MQQueueManager( argv[2] );
		}
		else
		{
			// queue name provided - use default queue manager
			mqQMgr = new MQQueueManager();
		}
	}
	catch (MQException* mqe) 
	{
		// stop if failed
		System::Console::WriteLine( S"create of MQQueueManager ended with {0}", mqe->Message );
		return( (int)mqe->Reason );
	}
			
	///
	/// Try to open the queue
	///
	try 
	{
		// If queue name provided, then use it
		// otherwise use the default queue name
		if (argc > 1) 
		{
			queueName = argv[1];
		}
		else 
		{
			queueName = EXISTING_QUEUE;
		}
		// The named queue may be a model queue, which will result in the
		// creation of a temporary dynamic queue, which will be destroyed as
		// soon as it is closed. Therefore we must ensure that such a queue
		// is not automatically closed and re-opened. We do this by setting
		// open options which will avoid the need for closure and re-opening.
		mqQueue = mqQMgr->AccessQueue( queueName, 
			MQC::MQOO_OUTPUT | MQC::MQOO_INPUT_SHARED | MQC::MQOO_INQUIRE );
	}
	catch (MQException* mqe) 
	{
		// stop if failed
		System::Console::WriteLine( S"MQQueueManager::AccessQueue ended with {0}", mqe->Message );
		return( (int)mqe->Reason );
	}

	///
	/// Prepare a message containing the text "Hello world".
	/// 
	mqMsg = new MQMessage();
	mqMsg->WriteString( MESSAGE );
	mqMsg->Format = MQC::MQFMT_STRING;
	mqPutMsgOpts = new MQPutMessageOptions();

	///
	/// Place the message on the queue, using default put message options.
	/// 
	try
	{
		mqQueue->Put( mqMsg, mqPutMsgOpts );
	}
	catch (MQException* mqe)
	{
		// report the error
		System::Console::WriteLine( S"MQQueue::Put ended with {0}", mqe->Message );
	}
	
    ///
	/// Discover the queue manager's name.
	/// 
	String* name = mqQMgr->Name;
	if (name != NULL) 
	{ 
		System::Console::WriteLine( S"The queue manager name is '{0}'.", name );
	}

	///
	/// Show the name of the queue.
	/// 
	name = mqQueue->Name;
	if (name != NULL) 
	{
		System::Console::WriteLine( S"Message sent to '{0}'.", name );
	}

	///
	/// Retrieve the data message just sent ("Hello world" expected) from the
	/// queue, using default get message options. We will get the message just
	/// sent, rather than any other message on the queue, because the "put" 
	/// will have set the id of the message in the message object, so, as we 
	/// are using the same message object, the message id acts as a filter 
	/// which says that we are only interested in a message if it has this 
	/// particular id.
	/// 
	mqGetMsgOpts = new MQGetMessageOptions();
	try 
	{
		mqQueue->Get( mqMsg, mqGetMsgOpts );

		// Show the text of the received message.
		System::Console::WriteLine( S"Message of length {0} bytes received, ", 
			                        __box(mqMsg->MessageLength) );

		if (String::Compare(mqMsg->Format, MQC::MQFMT_STRING->Substring(0, mqMsg->Format->Length)) == 0) 
		{
			System::Console::WriteLine(  mqMsg->ReadString(mqMsg->MessageLength) );
		}
		else 
		{
			System::Console::WriteLine( S"Non-text message" );
		}
	}
	catch (MQException* mqe) 
	{
		// report the error
		System::Console::WriteLine( S"MQQueue::Get ended with {0}", mqe->Message );
	}

	System::Console::WriteLine( S"Sample MMQWRLD end" );
	return (0);
}